//===-- Support/DotGraphTraits.h - Customize .dot output -------*- C++ -*--===//
//
// This file defines a template class that can be used to customize dot output
// graphs generated by the GraphWriter.h file.  The default implementation of
// this file will produce a simple, but not very polished graph.  By
// specializing this template, lots of customization opportunities are possible.
//
//===----------------------------------------------------------------------===//

#ifndef SUPPORT_DOTGRAPHTRAITS_H
#define SUPPORT_DOTGRAPHTRAITS_H

#include <string>

/// DefaultDOTGraphTraits - This class provides the default implementations of
/// all of the DOTGraphTraits methods.  If a specialization does not need to
/// override all methods here it should inherit so that it can get the default
/// implementations.
///
struct DefaultDOTGraphTraits {
  /// getGraphName - Return the label for the graph as a whole.  Printed at the
  /// top of the graph.
  ///
  static std::string getGraphName(void *Graph) { return ""; }

  /// getNodeLabel - Given a node and a pointer to the top level graph, return
  /// the label to print in the node.
  static std::string getNodeLabel(void *Node, void *Graph) { return ""; }

  /// If you want to specify custom node attributes, this is the place to do so
  ///
  static std::string getNodeAttributes(void *Node) { return ""; }

  /// If you want to override the dot attributes printed for a particular edge,
  /// override this method.
  template<typename EdgeIter>
  static std::string getEdgeAttributes(void *Node, EdgeIter EI) { return ""; }

  /// getEdgeSourceLabel - If you want to label the edge source itself,
  /// implement this method.
  template<typename EdgeIter>
  static std::string getEdgeSourceLabel(void *Node, EdgeIter I) { return ""; }

  /// edgeTargetsEdgeSource - This method returns true if this outgoing edge
  /// should actually target another edge source, not a node.  If this method is
  /// implemented, getEdgeTarget should be implemented.
  template<typename EdgeIter>
  static bool edgeTargetsEdgeSource(void *Node, EdgeIter I) { return false; }

  /// getEdgeTarget - If edgeTargetsEdgeSource returns true, this method is
  /// called to determine which outgoing edge of Node is the target of this
  /// edge.
  template<typename EdgeIter>
  static EdgeIter getEdgeTarget(void *Node, EdgeIter I) { return I; }
};


/// DOTGraphTraits - Template class that can be specialized to customize how
/// graphs are converted to 'dot' graphs.  When specializing, you may inherit
/// from DefaultDOTGraphTraits if you don't need to override everything.
///
template <typename Ty>
class DOTGraphTraits : public DefaultDOTGraphTraits {};

#endif
